(function () {
  var main = null;
  var modules = {
      "require": {
          factory: undefined,
          dependencies: [],
          exports: function (args, callback) { return require(args, callback); },
          resolved: true
      }
  };
  function define(id, dependencies, factory) {
      return main = modules[id] = {
          dependencies: dependencies,
          factory: factory,
          exports: {},
          resolved: false
      };
  }
  function resolve(definition) {
      if (definition.resolved === true)
          return;
      definition.resolved = true;
      var dependencies = definition.dependencies.map(function (id) {
          return (id === "exports")
              ? definition.exports
              : (function () {
                  if(modules[id] !== undefined) {
                    resolve(modules[id]);
                    return modules[id].exports;
                  } else return require(id)
              })();
      });
      definition.factory.apply(null, dependencies);
  }
  function collect() {
      Object.keys(modules).map(function (key) { return modules[key]; }).forEach(resolve);
      return (main !== null) 
        ? main.exports
        : undefined
  }

  define("src/enum-type", ["require", "exports"], function (require, exports) {
      "use strict";
      var ParamType;
      (function (ParamType) {
          ParamType[ParamType["PROPERTY"] = 0] = "PROPERTY";
          ParamType[ParamType["VALUE"] = 1] = "VALUE";
      })(ParamType = exports.ParamType || (exports.ParamType = {}));
      ;
  });
  define("src/origin-rules-analyse", ["require", "exports", "src/enum-type"], function (require, exports, enum_type_1) {
      "use strict";
      function originRulesAnalyse(originRules) {
          var realRules = {};
          for (var p in originRules) {
              var ruleStr = originRules[p];
              updateRule(realRules, p, ruleStr);
          }
          return realRules;
      }
      exports.originRulesAnalyse = originRulesAnalyse;
      function updateRule(rules, propName, propValue) {
          var ruleStringList = propValue.split(/\s*\|\s*/), selectorSplit = propName.split(/\s*:\s*/), field = selectorSplit[0];
          if (rules[field] === undefined) {
              rules[field] = [];
          }
          var ruleItem = rules[field];
          for (var i = 0, len = ruleStringList.length; i < len; i++) {
              var ruleStringItem = ruleStringList[i].split(/\s*:\s*/), ruleParams = void 0, methodName = ruleStringItem.splice(0, 1)[0].replace(/^\s+|\s+$/g, ''), isReverse = false;
              if (ruleStringItem.length > 0) {
                  ruleParams = ruleStringItem.map(function (item) {
                      var type;
                      if (/^\s*['"]|['"]\s*$/.test(item) || /^-?\d+(?:\.\d+)?$/.test(item)) {
                          type = enum_type_1.ParamType.VALUE;
                      }
                      else {
                          type = enum_type_1.ParamType.PROPERTY;
                      }
                      return {
                          type: type,
                          value: item.replace(/^\s+|\s+$/g, '').replace(/^['"]|['"]$/g, '')
                      };
                  });
              }
              if (methodName.indexOf('!') !== -1) {
                  isReverse = true;
                  methodName = methodName.substr(1);
              }
              ruleItem.push({
                  method: methodName,
                  params: ruleParams,
                  reverse: isReverse
              });
          }
      }
  });
  define("src/build-in-validators", ["require", "exports"], function (require, exports) {
      "use strict";
      exports.ValidatorCollection = {
          create: function () {
              return {
                  'required': function (value) {
                      return !(value === undefined || value === null || /^\s*$/.test(value));
                  },
                  'number': function (value) {
                      return /^-?\d+(?:\.\d+)?$/.test(value);
                  },
                  'email': function (value) {
                      return /^(\w+|\.+)((-\w+|\.+)|(\.+\w+))*\@\w+((\.|-)\w+)*\.\w+/.test(value);
                  },
                  'maxLen': function (value, len) {
                      var str = String(value), rps_value = str.replace(/^\s+|\s+$/g, '');
                      return rps_value.length <= len;
                  },
                  'eq': function (value, compareVal) {
                      return value == compareVal;
                  },
                  'gt': function (val1, val2) {
                      return val1 > val2;
                  },
                  'gte': function (val1, val2) {
                      return val1 >= val2;
                  },
                  'lt': function (val1, val2) {
                      return val1 < val2;
                  },
                  'lte': function (val1, val2) {
                      return val1 <= val2;
                  }
              };
          }
      };
  });
  define("src/rules", ["require", "exports", "src/origin-rules-analyse", "src/build-in-validators"], function (require, exports, origin_rules_analyse_1, build_in_validators_1) {
      "use strict";
      function rules(config) {
          var realRules = origin_rules_analyse_1.originRulesAnalyse(config), newValidators = build_in_validators_1.ValidatorCollection.create(), ruleFn = getRuleFunction(realRules, newValidators);
          ruleFn.register = function (methodName, fn) {
              newValidators[methodName] = fn;
          };
          return ruleFn;
      }
      exports.rules = rules;
      function getRuleFunction(realRules, newValidators) {
          return function (data) {
              for (var fieldName in realRules) {
                  var filedItem = realRules[fieldName], dataItem = data[fieldName];
                  for (var _i = 0, filedItem_1 = filedItem; _i < filedItem_1.length; _i++) {
                      var ruleItem = filedItem_1[_i];
                      var params = void 0;
                      if (ruleItem.params) {
                          params = ruleItem.params.map(function (param) {
                              return param.value;
                          });
                      }
                      var valid = newValidators[ruleItem.method].apply(null, [dataItem].concat(params));
                      if (!valid) {
                          return { valid: false };
                      }
                  }
              }
              return { valid: true };
          };
      }
  });
  define("index", ["require", "exports", "src/rules"], function (require, exports, rules_1) {
      "use strict";
      exports.rules = rules_1.rules;
  });
  
  return collect(); 
})();

rules({})